const char *version="m_1D.c v0.8 JS 2001-2019";
/*   
   creation of model files for 1D models from unitcells (see below)
   
   compile: gcc -DModel=5 -O2 -o exe/m_1d_sawtooth -static models/m_1d.c
            gcc -DModel=7 -O2 -o exe/m_1d_KagoMix  -static models/m_1d.c
   run:     cd exe;./m_1d_sawtooth 15 > daten.def
            ../utils/def2fig.sh -i daten.def > out.fig # xfig out.fig

  v0.8 add bond filter bitmask option -b<bitmask> (2019-04)     
  v0.7 add model 7 + output modelname  (2018-08)
  v0.6 resolve w7 u7 conflict of J1J2Chain (2017?)
  v0.5 cosmetic + bond vector R (buggy, fix 2019-03)
  v0.4 make D=1 lattices from one unitcell
  v0.3 J1J2-chain added, p_t set
  
  ToDo19: use zero-terminated arrays like m_lattice.c 
                       
*/
#include <stdlib.h>
#include <stdio.h>

/* old: exactly one of the following models must be set to defined */
/* new: every model is presented by a number and selected below */
#define PChain         1     /* plaquette chain, mixed spin 1/2 vs. 1 */
#define J1J2Chain      2
#define J1J2J3Chain    3
#define KagomeChain    4
#define TriangleChain  5     /* sawtooth or delta chain */
#define DiamondChain   6
#define MixedSpinKago  7     /* mixed spin Kagome chain 2018-08 */

/* please select one of the above defined models here */
/*    can be definded with gcc option: -DModel=2 */
#ifndef Model  /* may be defined by outside (make or cc option) */
#define Model J1J2Chain    /* example: 2 == J1J2Chain (default) */
#endif  /* ifndef Model */

/* S=1-S=1/2-P-chain  s2+s3=spin-1 s0+s1=plaquette (ToDo?)
 *                           like spin-1 chain where every 2nd spin has inner frustration
 *   3      1---3---1--
 *   2      | X   X | X
 *   1      0---2---0--
 *   0123
 *  
 */
#if Model == PChain
#define NUMsites 4
#define NUMbonds 9 
#define NUMidx   2
const char *modelname="PChain"; /* original name? */
int evec[2]={3,99};
int sites[NUMsites][2]={{1,1},{1,2},{2,1},{2,2}};  /* 4 sites */
int bonds[NUMbonds][4]={ /* site1 site2 idx-J cell-x */
 {0,2,1,0}, {0,3,1,0}, {1,2,1,0}, {1,3,1,0},
 {2,0,1,1}, {2,1,1,1}, {3,0,1,1}, {3,1,1,1},
 {0,1,2,0}};
/* ToDo19: use 0 terminated arrays, not macros, see m_lattice.c */
#endif


/* J2=2J1
 *   0===0===0===0=== J2
 *    \ / \ / \ / \ / J1
 *   ==0===0===0===0= J2
 * N=8 nud=4,4 sym_ud=+1 sym_k=-2 0 0  j1=1 j2=0
 *  q_0_0+ n1=7  -3.65109341  -1.80193774  -0.72610945  -0.44504187
 *  q_0_1+ n1=0
 *  q_4_0+ n1=5  -2.69962815  -0.76087672   0.00000000   1.00000000   1.46050487
 *  q_4_1+ n1=1   0.00000000
 *  q_0_0- n1=1   0.00000000
 *  q_0_1- n1=2  -1.00000000   0.00000000
 *  q_4_0- n1=0
 *  q_4_1- n1=4  -3.12841906  -1.20163968   0.00000000   1.33005874
 * testcase s=1 via (./m_1d 8 | ../utils/defspin1.sh -s 2 > m_s1.def)
 * nud=8,8 sym_ud=+1 sym_k=-2 0 0  j1=1 j2=0
 *  q_0_0+ n1=59 i025= -11.33695608  -9.59655995 SS= 6.000 2S= 4.000
 */
#if Model == J1J2Chain
#define NUMsites 1
#define NUMbonds 2
#define NUMidx   2
const char *modelname="J1J2Chain"; /* frustrated chain */
int evec[2]={1,99};
int sites[NUMsites][2]={{1,1}}; /* (x,y)-coord */
int bonds[NUMbonds][4]={ /* site1 site2 idx-J cell-x */
 {0,0,1,1},  {0,0,2,2}};
#endif

/* JS 2011 for playing with matrix.pgm */
#if  Model == J1J2J3Chain
#define NUMsites 1
#define NUMbonds 3   /* num bonds from i1 to i2 per unit cell */
#define NUMidx   3   /* maximum parameter index, 3 for J1-J2-J3 */
const char *modelname="J1J2J3Chain"; /* J3 chain */
int evec[2]={1,99};
int sites[NUMsites][2]={{1,1}}; /* x,y of site 0 */
int bonds[NUMbonds][4]={ /* site1 site2 idx-J cell-x */
 {0,0,1,1},  {0,0,2,2}, {0,0,3,3}};
#endif

/* J2=(3/2)J1  see below for MixedSpinKago
 * y=3  ------0---1----5---6---- J2
 *             \ /      \ /     J1
 * y=2          2        7
 *             / \      / \     J1
 * y=1  ------3---4----8---9---- J2
 *    
 *    x:      1 2 3    4 5 6
 */
#if Model ==  KagomeChain
#define NUMsites 5
#define NUMbonds 8
#define NUMidx   2
const char *modelname="Kagome-Chain"; /* J1-J2-Kago chain */
int evec[2]={4,99};
int sites[NUMsites][2]={{1,3},{3,3},{2,2},{1,1},{3,1}}; /* (x,y)-coord */
int bonds[NUMbonds][4]={ /* site1 site2 idx-J cell-x */
 {0,1,1,0},  {3,4,1,0}, {0,2,1,0},  {2,4,1,0},
 {1,2,1,0},  {2,3,1,0}, {1,0,2,1},  {4,3,2,1}};
#endif

/* 2018-08 MixedSpinKagomeChain  see above for KagomeChain
 * y=4    ----0---1----6---7----   J2
 *             \ /      \ /       J1
 * y=2,3       4,5     10,11
 *             / \      / \       J1
 * y=1    ----2---3----8---9----   J2
 *    
 *    x:      1 2 3    4 5 6
 *
 *  NN=36=6uc*6/uc j1=1 j2=0.94 nud=32+4 maxscfg=0x010400410
 *     sym_k=-999999     n1=29856 i080= 4.39752743 4.55803503
 *     sym_k=-3 0 0 -1 0  n1=1331 i055= 4.39752743 4.77046516
 *       ZMag= 0.15123457 XYMag= 0.07002977 SMag= 0.14053335
 *  NN=18=3uc*6/uc j1=1 j2=0.94 nud=16+2 maxscfg= 0x00410
 *     sym_k=-3 0 0 -1 0  n1=14   i016=   2.49792384   3.06000016
 *     sym_k=-3 0 -999999 n1=36   i038=   2.49792396   2.99347302
 *     sym_k=-3 -999999  n1=108   i040=   2.25865387   2.49792397
 *   2018-08 triggers problem a32 to small n1=48, ToDo: fix
 *     0x00009 to 0x0000c, why 0x0000a (s3+s1) skipped
 */
#if Model ==  MixedSpinKago
#define NUMsites 6
#define NUMbonds 12   /* num bonds from site i1 to site i2 per unit cell */
#define NUMidx   2    /* maximum parameter index, 2 for J1-J2 */
const char *modelname="MixedSpinKagome-Chain"; /* ri2018 origin name? */
int evec[2]={4,99};
int sites[NUMsites][2]={{1,4},{3,4},{1,1},{3,1},{2,2},{2,3}}; /* (x,y)-coord */
int bonds[NUMbonds][4]={ /* site1 site2 idx-J cell-shift_r2.x */
 {0,4,1,0},  {1,4,1,0}, {2,4,1,0},  {3,4,1,0},
 {0,5,1,0},  {1,5,1,0}, {2,5,1,0},  {3,5,1,0},
 {0,1,2,0},  {2,3,2,0}, {1,0,2,1},  {3,2,2,1}};
#endif

/* J2=2J1  sawtooth or triangle-chain
 * y=2    O   O   O         2016-12 add picture
 *       / \ / \ / \   J2
 * y=1  O===O===O===   J1
 *
 *    x=1 2
 */
#if Model == TriangleChain
#define NUMsites 2
#define NUMbonds 3
#define NUMidx   2
const char *modelname="Delta, Sawtooth or Triangle-Chain"; /* origin name? */
int evec[2]={2,99};
int sites[NUMsites][2]={{1,1},{2,2}};  /* (x,y)-coord */
int bonds[NUMbonds][4]={ /* site1 site2 idx-J cell-x */
 {0,1,2,0},  {1,0,2,1}, {0,0,1,1}};
#endif

#if Model == DiamondChain
/*
 *  y=3  -- 1 ----- 4'-----
 *        /   \   /   \   /
 *  y=2         2       5'
 *        \   /   \   /   \
 *  y=1  -- 0 ----- 3'-----
 *
 *        x=1   2   3   4
 *  
 */
#define NUMsites 3
#define NUMbonds 6
#define NUMidx   1
const char *modelname="Diamond-Chain"; /* origin name? */
int evec[2]={2,99};
int sites[NUMsites][2]={{1,1},{1,3},{2,2}}; /* (x,y)-coord */
int bonds[NUMbonds][4]={ /* site1 site2 idx-J cell-x */
 {0,2,1,0},  {1,2,1,0}, {0,0,1,1},
 {1,1,1,1},  {2,0,1,1}, {2,1,1,1}};
#endif

int initvars(int n0, int pbc, int bondmask){
   int i,j,jb,nn,nw,ucBonds=NUMbonds;
   if (bondmask) /* ToDo: bitfilter? */
   for(j=0,ucBonds=0;j<NUMbonds;j++) /* all bonds, cell-vector is simple */
     if ((bondmask<<1) & (1<<bonds[j][2])) ucBonds++; /* bitmask per bond idx */
   nn=n0*NUMsites;
   nw=n0*ucBonds;
   printf("# def-file, 1D-chain model=%d %s\n"
	"# automaticly generated by %s n0=%d PBC bmask=%0x\n"
	"#===edge===  %d 0  0 999\n"
	"#+++model+++\n"
	"# NN=Number_of_sites Nw=Number_of_bonds [Nq=quad_terms]\n"
	"%d %d 0\n"
	"# s=site  XYZ=position #=index e=one_site_energy fz=magnetic_field_z\n"
	"# s Rx Ry Rz [e#] [U#] [fz#]\n",
     Model, modelname, version, n0, bondmask, n0*evec[0], nn, nw);
   for(i=0;i<n0;i++)for(j=0;j<NUMsites;j++)
     printf(" %2d %2d %2d  0 e8 u7\n",
        i*NUMsites+j, sites[j][0]+i*evec[0], sites[j][1]);
   /* old2019: n=index s1,s2=sites R=PBCvector_to_UC t,j,v,w=parameters */
   /* new2019: n=index s1,s2=sites R=vector(Rs2-Rs1) t,j,v,w=parameters */
   /*  v0.5 vector R will help making good geometry images from def-file
    *    using Rx,Ry,Rz + edge_vec_matrix * cell_shift_R (modeldef.c)
    *  ToDo: better put Dx,Dy,Dz bond direction a la out.splot? 
    */
   printf("# Bond s1 s2 [R= # # #] [t#] [j#] [v#] [w#]\n");
   for(jb=0,i=0;i<n0;i++) /* for all cells = $length * unit_cell */
   for(j=0;j<NUMbonds;j++) /* all bonds, cell-vector is simple */
   if (bondmask==0 || ((bondmask<<1) & (1<<bonds[j][2]))) { /* big bond filter 2019-04 */
     printf(" %2d %2d %2d  R= %2d %2d %2d  t%d j%d v0 w%d %c\n",
      jb,            /* bond or pair index old: i*NUMbonds+j */
      bonds[j][0]+NUMsites*i,  /* site 1 of the pair */
      bonds[j][1]+NUMsites*((i+bonds[j][3]+n0)%n0), /* site 2 of the pair */
    /*  i, */      /* R_cell_shift.x, y=z=0 for chain, 2019-03 fix ??? */
      /* 2019-09-20 fix, geometr. bond-vector for 2D-plotting */
      sites[bonds[j][1]][0] - sites[bonds[j][0]][0] + bonds[j][3]*evec[0],
      sites[bonds[j][1]][1] - sites[bonds[j][0]][1] +           0*evec[1],
      0, /* no z-components */
      bonds[j][2]+NUMidx,  /* t3 t4 */
      bonds[j][2],         /* J1 J2 */
      bonds[j][2]+2*NUMidx, /* w5 w6, 2014-05-22 w7 and u7 conflicts */ 
      (((i+bonds[j][3]>=n0 || i+bonds[j][3]<0) && pbc==1)?'+':' ') );
     jb++;
   } // bondmask + bond-loop j
   printf("#---model---\n");
   return 0;
} // initvars

int help(int rc){
  fprintf(stderr,
    " generate daten.def for an 1D chain, %s\n"
    "   compiled with option Model=%d for %s\n"
    " use: ./m_1D [-b<1..9>] <length>\n"
    "   -b<bondmask>: bond filter bitmask, 1=J1 2=J2 3=J1+J2 4=J3 ...\n"
    "   length: 1..64  chain length in unit cells\n",
    version, Model, modelname);
  exit(rc);
} // help

int main(int argc,char *argv[]){
  int ilen,       /* length in unitcells */
      bondmask=0, /* bond filter mask, 1=J1 2=J2 3=J1+J2 4=J3 */
      iarg=1;     /* argument index, 0 is command name, 1 is arg1 */
  while (argc-iarg>=1 && argv[iarg][0]=='-') {
    switch (argv[iarg][1]) {
      case 0x00: {iarg++;break;}; /* - only */
      case 'b': { bondmask= atoi(argv[iarg]+2);break; };
      default: help(2);
    }
    iarg++;
  }
  if (argc-iarg!=1) help(1);
  ilen = atoi(argv[iarg]); if (ilen==0) help(3);
  fprintf(stderr,"DBG: bondmask=%0x len=%d\n",bondmask,ilen);
  initvars(ilen,1,bondmask);
  return 0;
} // main
